/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2016,2019
 */
Polymer({
	is: 'maximo-submitsr-location',
  	behaviors: [BaseComponent,InfiniteScroll,HTMLEncoder,ServiceRequestBehavior],
    properties: {
		label: {
			type: String,
			value: 'Panel'
		},		
		title: {
			type: String,
			value: 'title'
		},
		recordData :{ //object for location list data
			type:Array,
			value:[],
			notify : true
		},
		locanData :{ // object for location ancestor list data
			type : Object,
			notify : true
		},
		locancestor :{ // array for location ancestor list
			type: Array,
    		value: []
		},
		selectedRecord :{
			type:Object,
			notify:true
		},
		additionalparams :{
			type : Object,
			notify : true
		},
		additionalancestorparams :{
			type : Object,
			notify : true
		},
		selectedItem : {
			type : Object
		},
		selectedQueryName :{
			type : Object,
			notify : true
		},
		bcdata: { // array for listing breadcrumb
    		type: Array,
    		value: []
    	},
    	overlayOpened: {
			type: Boolean,
			value: false
		},
		selectedLocationDesc :{
			type: String
		},
		selectedBc :{ // Selected location on breadcrumb
			type: String,
			value : ''
		},
		selectedLocation :{ 
			type:String
		},
		selectedSiteid : { // Selected siteid of location
			type: String,
			value : ''
		},
		assetSiteid : { // If asset step is first, save asset info
			type: String
		},		
		searchValue: {
            type: String,
            value: '',
            notify: true
        },
        userLocationDesc:{ // location description of user location
        	type : String
        },
        userLocation:{ // location of user 
        	type : String
        },
        systemid :{ // systemid of location hierarchy
        	type: String
        },
        systemType:{ // system type of location hierarchy
        	type : Array,
        	value : []
        },
        systemsFilter:{
        	type : Array,
        	value : []
        },
        assetdesc :{ 
        	type : String
        },
        assetnum :{
        	type : String
        },
        locationFirst:{ // if false, asset step is before the location step
        	type:Boolean,
        	value : false
        },
        lookupWhereClause :{
			type : String
    	},
		lookupLabels : {
			type : Object,
			value : []
		},
		showintro:{ // if false, asset step is before the location step
	        	type:Boolean,
	        	value : true
		},
		radiusValue:{
			type: Number
		},
		 /**
      	 * Indicates if the screen is on desktop mode
      	 */
		isDesktop: {
			type: Boolean,
			value: true,
			observer: '_updateCSS'
		},
		selectedOrgid : {
			type: String
		},
		assetOrgid : {
			type: String
		},	
		
		 
	},
	listeners :{
		'dom-change' : 'showNodataLabel',
		'maximo-lookup-selected' : '_returnFromLookup',
  		'maximo-typeahead-selected' : '_returnFromLookup',
  		'maximo-qrcode-result' : '_returnQRcode',
	},
	ready: function(){
		this.$.locationRadiusValue.fetchProperty();
		var self = this;
  		$j(window).on('resize', function(){
			 self._windowResize();
	    });
	},
	attached : function(){
		try {
			 //check which step is first
			var locIdx, assIdx;
			var steps = this.parentNode.children;
			for(var i=0; i<steps.length; i++) {
				if(steps[i].tagName === 'MAXIMO-SUBMITSR-ASSET') {
					assIdx = i;
				}
				else if(steps[i].tagName === this.tagName) {
					locIdx = i;
				}
			}
			this.locationFirst = locIdx > assIdx ?  false : true;
			//check if user's location is specified
			if($M.userInfo && $M.userInfo.location && $M.userInfo.location.location){
				this.userLocation = $M.userInfo.location.location;
				if($M.userInfo.location.description){
					this.userLocationDesc = $M.userInfo.location.description;
				}
				else{
					this.userLocationDesc = this.userLocation;
				}
			}

			this.lookupWhereClause = this._getLookupWhereClause();
			this.lookupLabels = [
				undefined,
				$M.localize('uitext', 'mxapisr', 'Parent')
			];
			var self = this;
			this.async(function(){
				self.$.locationListTemplate.scrollTarget=self.$.locationList; 
				self.infiniteScrollingList(this.$.locationListTemplate, 'locationcollection');
				self._windowResize();
			}, 100);
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	
	/**
	 * set the scree to desktop / mobile mode
   	 */
	_windowResize : function(){
		this.isDesktop = !$M.isMobile();
	},
	
	/**
	 * set the scree to desktop / mobile mode
   	 */
	_updateCSS : function(){
		try {
			$j(this.$.locationList).toggleClass('mobile',!this.isDesktop);
			$j(this.$.setLocationButtonWrapper).toggleClass('mobile',!this.isDesktop);
		} catch(err) {
			this._displayUnexpectedError(err);
		}
	
		
	},
	
	/**
	 * Go to the next page 
	 */
	_goNext : function(e) {
		try {
			 // if user selects his or her location
			if(this.userLocation === this.selectedLocation && !this.selectedLocationDesc.includes($M.localize('uitext','mxapisr','YourLocation'))){
				this.selectedLocationDesc += ' ('+$M.localize('uitext','mxapisr','YourLocation')+')';
			}
			// check if asset is selected in asset step
			if(!this.assetnum){
				if(this.locationFirst){
					this.fire('go-nextdetail', [
						{step:'Location', value: {description: this.selectedLocationDesc, breadcrumb: this.selectedBc, location:this.selectedLocation, selectedinlocation:true, 
							siteid : this.selectedSiteid, orgid: this.selectedOrgid}},
						{step:'Asset'}]);
				}
				else{ // asset is before the location step
					this.fire('go-nextdetail', [{step:'Location', value: {description: this.selectedLocationDesc, breadcrumb: this.selectedBc, location:this.selectedLocation, selectedinlocation:true, 
						siteid : this.selectedSiteid, orgid: this.selectedOrgid}},
						{step:'Asset', value:{}}]);
				}
			}
			else{
				this.fire('go-nextdetail', [{step:'Location', value: {description: this.selectedLocationDesc, breadcrumb: this.selectedBc, location:this.selectedLocation, siteid : 
					this.selectedSiteid, skipped:false, selectedinlocation:true, orgid: this.selectedOrgid}},
					{step:'Asset',value: {description: this.assetdesc, assetnum : this.assetnum, siteid : this.assetSiteid, orgid: this.assetOrgid}}]);
			}
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	/**
	 * Redirect on next page when skipped
	 */
	_skipStep: function(e) {
		try {
			this.selectedLocation = null;
			this.selectedBc = null;
			this.selectedLocationDesc = null;
			this.$.locSearch.value = null;
			  this.async(function(){
				  this.$.locSearch._closeTAResult(e);
			});
			this.fire('go-nextdetail', {step:'Location', value: {}});
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	/**
	 * Initialize the page data
	 */
	initPage: function() {
		try {
			this.selectedLocation = null;
			this.selectedBc = null;
			this.selectedLocationDesc = null;
			this.assetdesc = null;
			this.assetnum = null;
			this.assetSiteid = null;
			this.assetOrgid = null;
			this.additionalparams = [];
			this.bcdata = [];
			this.$.locSearch.value=null;
			this.$.locSearch._lookupObject.additionalLookupParams= this._getLookupWhereClause();
			this.lookupWhereClause = this._getLookupWhereClause();
			this.lookupLabels = [
				undefined,
				$M.localize('uitext', 'mxapisr', 'Parent')
			];
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	/**
	 * function for reaching parent location of user's location.
	 */
	getToUserLocation : function(){
		try {
			 this.selectedQueryName = 'SERVICEREQUESTLOCATION';
			 this.additionalparams = [];
			 var selected = this.userLocation;

			  this.additionalparams.push('oslc.where=location=%22'+encodeURIComponent(selected)+'%22'+this.getDefaultSiteidParams(true));
			  this.additionalparams.push('responseos=MXAPILOCATION');
			  this.additionalparams.push('ignorecollectionref=1');
			  var that = this;
			  this.$.locationcollection.refreshRecords().then(function(collection) {
				  if(that.recordData.length == 0){
					  that.userLocation ='';
					  that._configureLocationList();
					  return;
				  }
				  
				  that.selectedQueryName = 'SERVICEREQUESTLOCATION';
				  that.selectedItem = {};
				  if(that.recordData[0].locations[0].description){
					  that.userLocationDesc = that.recordData[0].locations[0].description;
				  }
				  else{
					  that.userLocationDesc = that.recordData[0].location;
				  }
				  
				  that.selectedSiteid = that.recordData[0].siteid;
				  
				  //that.initUserLocationBreadcrumb(that.locanestor, that.recordData[0]);
				  if(!that.systemid){
					  that.systemid = that.recordData[0].systemid;
					  if(that.recordData[0].parent){
						  that.addParams(that.recordData[0].parent);
					  }
				  }
				  else{
					  if(that.recordData.length==1 &&  that.recordData[0].parent){
						  that.addParams(that.recordData[0].parent);
					  }else{
						  for(var i=0; i<that.recordData.length;i++){
							  if(that.systemid === that.recordData[i].systemid){
								  if(that.recordData[i].parent){
									  that.addParams(that.recordData[i].parent);
								  }
								  else{ // when user location is root
									  
									  that.$.setLocationButton.hidden = true;
									  that.$.breadcrumb.hidden = true;
									  $j(that.$.indoorList).toggleClass('setLocation',false);
									  $j(that.$.setLocationButtonWrapper).toggleClass('setLocation',false);
									  that.selectedQueryName = 'SERVICEREQUESTROOTLOCATION';
									  that.additionalparams = [];
									  
									  that.additionalancestorparams = [];
									  
									  that.additionalparams.push('ignorecollectionref=1');
									  that.$.locationcollection.refreshRecords().then(function(collection) {
										  that.setYourLocationLabel(that.userLocationDesc);
										  that.$.indoorList.hidden = false;
										  that.$.breadcrumb.hidden = true;
										  that.$.loadingSpinner.hidden = false;
									  });
								  }
							  }
						  }
					  }
				  }
			  });
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
  		
	},
	/**
	 * Sorting the information label in the list
	 */
	_computeSort: function(a, b) {
		try {
			 //user's location should be on the top of list
			if(a.location === this.userLocation && b.location !== this.userLocation){
				return -1;
			}
			else if(a.location !== this.userLocation && b.location === this.userLocation){
				return 1;
			}
			var loca, locb;
			loca = a.locations[0].description ? a.locations[0].description : a.location;
			locb = b.locations[0].description ? b.locations[0].description : b.location;
			if(loca === locb){
				a = null;
			}
			else{
				return loca < locb ? -1 : 1;
			}
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	    
    },
    /**
	 * Set your location label
	 */
	setYourLocationLabel : function(desc){
		try {
			for(var i=0; i<this.recordData.length; i++){
				if(this.querySelectorAll('.locationLabel')[i].label === desc){
					this.querySelectorAll('.locationLabel')[i].label = desc+' ('+$M.localize('uitext','mxapisr','YourLocation')+')';
					break;
				}
			}
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	/**
	 * function for setting breadcrumb if user's location is specified at first
	 */
	initUserLocationBreadcrumb : function(list){
		try {
			this.bcdata = [];
			var tempParent;
			var userloc = this.userLocation;
			//building location hierarchy
			for(var i=0; i<list.length; i++){
				if(list[i].ancestor === userloc){ 
					if(userloc !== this.userLocation){
						this.bcdata.push({'description':list[i].locations.description, 'location':list[i].ancestor});
					}
					if(list[i].lochierarchy.parent){
						tempParent = list[i].lochierarchy.parent;
					}
					else{
						break;
					}
					for(var j =0 ; j<list.length; j++){
						if(list[j].ancestor === tempParent){
							this.bcdata.push({'description':list[j].locations.description, 'location':list[j].ancestor});
							userloc = list[j].lochierarchy.parent;
							i = -1;
							break;
						}
					}
				}
			}
			this.bcdata.push({'description':$M.localize('uitext','mxapisr','all'), 'location':$M.localize('uitext','mxapisr','all')});
			this.bcdata.reverse();
			
			this.selectedLocation = this.bcdata[this.bcdata.length-1].location;
			this.selectedLocationDesc = this.bcdata[this.bcdata.length-1].description;
			this.setBreadCrumb();
			this.$.breadcrumb.hidden = false;
			this.locancestor = [];
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	/**
	 * Handle record data
	 */
	_handleRecordDataRefreshed : function(){
		this.searchValue='';
	},
	/**
	 * Handle data error
	 */
	_handleDataError : function(){
	},
	
	/**
	 * Get Asset siteid parameters
	 */
	getAssetSiteidParams : function(){
		try {
			var param = '';
			if(this.systemType.length === 0 && this.systemsFilter.length === 0){
				param+='siteid=%22'+this.assetSiteid+'%22';
			}
			else{
				param += '%20and%20siteid=%22'+this.assetSiteid+'%22';
			}
			return param;
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	/**
	 * Add parameters
	 */
	addParams : function(selected){
		try {
			this.additionalparams = [];
			var param = 'oslc.where=parent=%22'+encodeURIComponent(selected)+'%22'+this.getDefaultSiteidParams(true);
	//		 
			
			  this.additionalparams.push(param);
			 
			  this.additionalparams.push('responseos=MXAPILOCATION');
			  this.additionalparams.push('ignorecollectionref=1');
			  var that = this;
			  this.$.locationcollection.refreshRecords().then(function(collection) {
				  if(that.recordData.length === 0){
					  that.$.indoorList.hidden = true;
					  that.querySelector('.bcLast').ellipsisSize = 0;
					  that.querySelector('.bcLast').label = that.querySelector('.bcLast').getAttribute('locationdesc');
					  //$j(that.$.setLocationWrapper).addClass('setLocationWrapper_noData');
				  } else {
					  that.$.indoorList.hidden = false;
					  that.setYourLocationLabel(that.userLocationDesc);
					  //$j(that.$.setLocationWrapper).removeClass('setLocationWrapper_noData');
				  }
				  that.$.loadingSpinner.hidden = true;
			  });
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	/**
	 * Add Ancestor parameters
	 */
	addAncestorParams : function(location){ 
		try {
			this.additionalancestorparams = [];
			//this.recordData = [];
			var param = 'oslc.where=location=%22'+encodeURIComponent(location)+'%22'+this.getDefaultSiteidParams(true);
	//		 
			
			  this.additionalancestorparams.push(param);
			this.additionalancestorparams.push('responseos=MXAPILOCANCESTOR');
			this.additionalparams.push('ignorecollectionref=1');
			  var that = this;
			  this.$.locancestorcollection.refreshRecords().then(function(collection) {
				  var sysId = that.systemid;
				  for(var i=0; i<that.locanData.length; i++){
					  if(that.locanData[i].ancestor === that.userLocation){
						  if(!sysId){
							  that.systemid = sysId = that.locanData[i].systemid;
						  }
					  }
				  }
				  for(i=0; i<that.locanData.length; i++){
					  if(that.locanData[i].systemid === sysId){
						  that.locancestor.push(that.locanData[i]);
					  }
				  }
				  if(that.locancestor.length !== 1){
					  that.initUserLocationBreadcrumb(that.locancestor);
				  }
			  });
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	/**
	 * Get location label
	 */
	getLocationLabel : function(loc){
		try {
			if(loc.locations[0].description){
				return loc.locations[0].description;
			}
			else{
				return loc.location;
			}
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	/**
	 * Find record data
	 */
	findRecordData : function(desc){
		try {
			for(var i=0; i<this.recordData.length; i++){
				if(this.recordData[i].locations[0].description){
					if(this.recordData[i].locations[0].description === desc){
						return this.recordData[i];
					}
				}
				else{
					if(this.recordData[i].location === desc){
						return this.recordData[i];
					}
				}
			}	 
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	/**
	 * function for when user taps location item on the list
	 */
	getLocations : function(e){	
		try {
			e.preventDefault();
			this.$.loadingSpinner.hidden = false;
			var selected;
			var selectedName;
			var selectedSiteid = '';
			var selectedOrgid = '';
			
			this.$.breadcrumb.hidden =false;
	
			if(e.detail && e.detail.records && e.detail.records[0]){
				selected = e.detail.records[0].location;
				selectedName = e.detail.records[0].description;
				selectedSiteid = e.detail.records[0].siteid;
				selectedOrgid = e.detail.records[0].orgid;
			}
			
			this.selectedLocation = selected;
			this.selectedLocationDesc = selectedName;
			this.selectedSiteid = selectedSiteid;
			this.selectedOrgid = selectedOrgid;
			
			if(this.bcdata.length === 0){
				var first = {'location':$M.localize('uitext','mxapisr','all'), 'description':$M.localize('uitext','mxapisr','all')};
				this.bcdata.push(first);
				this.$.setLocationButton.hidden = false;
				$j(this.$.indoorList).toggleClass('setLocation',true);
				 $j(this.$.setLocationButtonWrapper).toggleClass('setLocation',true);
			}
			var locData;
			
			
			if(selected){
				locData = {'description': selectedName,'location':selected};
				
				var currentBCLocation = this.bcdata[this.bcdata.length-1].location;
				if(currentBCLocation!==selected){
					//only set bc if current bc is not selected location.
					this.bcdata.push(locData);
				}
				this.selectedLocation = selected;
				this.selectedLocationDesc = selectedName;
				
				this.selectedQueryName = 'SERVICEREQUESTLOCATION';
				this.selectedItem = {};
				if(selected){
					this.addParams(selected);
				}
				this.setBreadCrumb();
				return;
			}
			
			if(e.target.textContent.trim().includes($M.localize('uitext','mxapisr','YourLocation'))){
				selected = this.userLocation;
				selectedName = this.userLocationDesc;
				selectedSiteid = this.selectedSiteid;
				selectedOrgid = this.selectedOrgid;
			}
			else{
				var data = this.findRecordData(e.target.textContent.trim());
				selected = selectedName = data.location;
				selectedSiteid  = data.siteid;
				selectedOrgid = data.orgid;
				
			
				this.$.locSearch._lookupObject.additionalLookupParams=='responseos=MXAPIOPERLOC&savedQuery=SERVICEREQUESTLOCATION&oslc.where=locancestor.ancestor=%22'+encodeURIComponent(selected)+'%22'+this.getDefaultSiteidParams(true);
				
				
				if(data.locations[0].description){
					selectedName = data.locations[0].description;
				}
			}
			
			this.selectedLocation = selected;
			this.selectedLocationDesc = selectedName;
			this.selectedSiteid = selectedSiteid;
			this.selectedOrgid = selectedOrgid;
			
			locData = {'description': selectedName,'location':selected};
			this.bcdata.push(locData);
			
			this.selectedQueryName = 'SERVICEREQUESTLOCATION';
			this.selectedItem = {};
			if(selected){
				this.addParams(selected);
			}
			this.setBreadCrumb();
		} catch(err) {
			this._displayUnexpectedError(err);
		}	
		
	},

	/**
	 * Get default siteid parameters
	 */
	getDefaultSiteidParams : function(includeAnd){
		try {
			 
			var param = '';
			if(this.defaultInsertSite){
				if(includeAnd){
					param  +='%20and%20siteid=%22'+HTMLEncoder.sanitize(this.defaultInsertSite)+'%22';
				}
				else{
					param  +='siteid=%22'+HTMLEncoder.sanitize(this.defaultInsertSite)+'%22';
				}
			}
			return param;
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},

	/**
	 * function for when user taps location item on breadcrumb
	 */
	getBcLocation : function(e){
		try {
			this.$.loadingSpinner.hidden = false;
			var locDesc = e.target.getAttribute('locationdesc');
			var that = this;
			var selected = null;
			var localizedAll = $M.localize('uitext','mxapisr','all');
			if(locDesc === localizedAll || e.target.textContent.trim() === localizedAll){
				this.bcdata = [];
				this.$.breadcrumb.hidden =true;
				this.selectedQueryName = 'SERVICEREQUESTROOTLOCATION';
				this.additionalparams = [];
				var param='';

				if(this.filterSite){
					
					if(this.defaultInsertSite){
						if(param !== ''){
							param += this.getDefaultSiteidParams();
						}
						else{
							param += 'oslc.where='+this.getDefaultSiteidParams();
						}
					}
				}else if(this.assetSiteid){
					if(param !== ''){
						param += this.getAssetSiteidParams();
					}
					else{
						param += 'oslc.where='+this.getAssetSiteidParams();
					}
				}
				this.$.locSearch._lookupObject.additionalLookupParams=this.lookupWhereClause;
				this.additionalparams.push(param);
				this.additionalparams.push('ignorecollectionref=1');
				this.$.setLocationButton.hidden = true;
				this.$.indoorList.hidden = false;
				$j(this.$.indoorList).toggleClass('setLocation',false);
				$j(this.$.setLocationButtonWrapper).toggleClass('setLocation',false);
				this.$.locationcollection.refreshRecords().then(function(collection) {
					if(that.userLocationDesc){
						that.setYourLocationLabel(that.userLocationDesc);
					}
					that.$.loadingSpinner.hidden = true;
				  });
			}
			else{
				for(var i=0; i<this.bcdata.length; i++){
					if(this.bcdata[i].description === locDesc){
						selected = this.bcdata[i].location;
						this.$.locSearch._lookupObject.additionalLookupParams= 'responseos=MXAPIOPERLOC&savedQuery=SERVICEREQUESTLOCATION&oslc.where=locancestor.ancestor=%22'+encodeURIComponent(selected)+'%22'+this.getDefaultSiteidParams(true);
					
						break;
						
					}
				}
				if(selected){
					this.addParams(selected);
				}
				for(i = this.bcdata.length-1; i>0; i--){
					if(this.bcdata[i].description === locDesc){
						break;
					}
					this.bcdata.pop();
				}
				this.setBreadCrumb();
			}
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	/**
	 * function for setting breadcrumb
	 */
	setBreadCrumb : function(last){
		try {
			var that = this;
			while ( this.$.breadcrumb.hasChildNodes() )
			{
				this.$.breadcrumb.removeChild(this.$.breadcrumb.firstChild);       
			}
			this.selectedBc = '';
			var localizedAll = $M.localize('uitext','mxapisr','all');
			if(this.bcdata.length <= 7){ // if number of location item is not more than 7 
				for(var i =0; i<this.bcdata.length; i++){
					if(i === this.bcdata.length-1){ // last location on breadcrumb
						var bcl = Polymer.Base.create('MAXIMO-LABEL', {'id':'bc'+i, 'label':this.bcdata[i].description});
						bcl.setAttribute('class', 'bcLast maximo-submitsr-location');
						bcl.setAttribute('locationdesc', HTMLEncoder.sanitize(this.bcdata[i].description));
						bcl.$.label.setAttribute('locationdesc', HTMLEncoder.sanitize(this.bcdata[i].description));
						bcl.ellipsisSize = 20;
						this.$.breadcrumb.appendChild(bcl);
						if(this.bcdata[i].description !== localizedAll){
							this.selectedBc += this.bcdata[i].description;
						}
					}
					else{
						var bc = Polymer.Base.create('MAXIMO-LABEL', {'id':'bc'+i, 'label':this.bcdata[i].description, 'icon':'image:navigate-next'});
						bc.setAttribute('class', 'bcList maximo-submitsr-location');
						bc.setAttribute('locationdesc', HTMLEncoder.sanitize(this.bcdata[i].description));
						bc.$.label.setAttribute('locationdesc', HTMLEncoder.sanitize(this.bcdata[i].description));
						bc.ellipsisSize = 20;
						bc.addEventListener('click', function(e){
							that.findLocationObjInBc(that.bcdata,e.target.parentNode.getAttribute('locationdesc'));
							that.getBcLocation(e);
						});
						this.$.breadcrumb.appendChild(bc);
						if(this.bcdata[i].description !== localizedAll){
							this.selectedBc += this.bcdata[i].description+' > ';
						}
					}
				}
			}
			else{ // if number of location item is more than 7
				var all = Polymer.Base.create('MAXIMO-LABEL', {'id':'bc'+0, 'label': $M.localize('uitext','mxapisr','all'),'icon':'image:navigate-next'});
				all.setAttribute('class', 'bcList maximo-submitsr-location');
				var elps = Polymer.Base.create('MAXIMO-LABEL', {'id':'bc1', 'label': '...','icon':'image:navigate-next'});
				elps.setAttribute('class', 'bcList maximo-submitsr-location');
				all.addEventListener('click', function(e){
					that.getBcLocation(e);
				});
				this.$.breadcrumb.appendChild(all);
				this.$.breadcrumb.appendChild(elps);
				
				this.$.bdsection.style.top = elps.offsetTop+23 +'px';
				this.$.bdsection.style.left = elps.offsetLeft-10 + 'px';
				while ( this.$.bdListWrapper.hasChildNodes()) // initialize list box on breadcrumb('...')
				{
					this.$.bdListWrapper.removeChild(this.$.bdListWrapper.firstChild);       
				}
				//setting list box on breadcrumb('...')
				for(var d = 2; d<this.bcdata.length-5; d++){
					var dbc = Polymer.Base.create('MAXIMO-LABEL', {'id':'dbc'+d, 'label':this.bcdata[d].description});
					dbc.setAttribute('class', 'dbcList maximo-submitsr-location');
					dbc.setAttribute('locationdesc', HTMLEncoder.sanitize(this.bcdata[d].description));
					dbc.$.label.setAttribute('locationdesc', HTMLEncoder.sanitize(this.bcdata[d].description));
					dbc.ellipsisSize = 20;
					dbc.addEventListener('click', function(e){
						that.getBcLocation(e);
						that.overlayOpened = false;
					});
					this.$.bdListWrapper.appendChild(dbc);
				}
				elps.addEventListener('click', function(e){
					that.overlayOpened = true;
				});
				for(var j = this.bcdata.length-5; j<this.bcdata.length; j++){
					
					if(j === this.bcdata.length-1){
						var leftbcl = Polymer.Base.create('MAXIMO-LABEL', {'id':'bc'+j, 'label':this.bcdata[j].description});
						leftbcl.ellipsisSize = 20;
						leftbcl.setAttribute('class', 'bcLast maximo-submitsr-location');
						leftbcl.setAttribute('locationdesc', HTMLEncoder.sanitize(this.bcdata[j].description));
						leftbcl.$.label.setAttribute('locationdesc', HTMLEncoder.sanitize(this.bcdata[j].description));
						this.$.breadcrumb.appendChild(leftbcl);
						if(this.bcdata[j].description !== localizedAll){
							this.selectedBc += this.bcdata[j].description;
						}
					}
					else{
						var leftbc = Polymer.Base.create('MAXIMO-LABEL', {'id':'bc'+j, 'label':this.bcdata[j].description, 'icon':'image:navigate-next'});
						leftbc.setAttribute('class', 'bcList maximo-submitsr-location');
						leftbc.ellipsisSize = 20;
						leftbc.setAttribute('locationdesc', HTMLEncoder.sanitize(this.bcdata[j].description));
						leftbc.$.label.setAttribute('locationdesc', HTMLEncoder.sanitize(this.bcdata[j].description));
						leftbc.addEventListener('click', function(e){
							that.findLocationObjInBc(that.bcdata,e.target.parentNode.getAttribute('locationdesc'));
							that.getBcLocation(e);
						});
						this.$.breadcrumb.appendChild(leftbc);
						if(this.bcdata[j].description !== localizedAll){
							this.selectedBc += this.bcdata[j].description+' > ';
						}
					}
				}
			}
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	/**
	 * function for finding location objects in breadcrumb
	 */
	findLocationObjInBc : function(list, desc){
		try {
			for(var i=0; i<list.length; i++){
				if(list[i].description === desc){
					this.selectedLocation =  list[i].location;
					this.selectedLocationDesc = list[i].description;
				}
			} 
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	
    /**
	 * function for showing nolabel when no data in the list
	 */
    showNodataLabel : function(){
		try {
			if(this.searchValue){
				if(!this.querySelector('.locationListWrapper')){
					this.$.nodataLabelWrapper.hidden=false;
				}
				else{
					this.$.nodataLabelWrapper.hidden=true;
				}
			}
			var that = this;
			setTimeout(function() {
				that._setLayout();
			}, 100);
		} catch(err) {
			this._displayUnexpectedError(err);
		}
    	
    	
    },
    /**
	 * Set layout for the data
	 */
    _setLayout : function(){
//    	if(this.querySelector('.indoorList').offsetHeight+this.$.breadcrumb.offsetHeight > window.innerHeight-361){ // fix buttons // 100 is height of banner and header
//    		if($M.screenInfo.device !=='desktop'){
//    			$j(this.querySelector('.setLocationWrapper')).addClass('setLocationButtonWrapperFixed');
//    			$j(this.querySelector('.skipButtonDiv')).addClass('skipbuttonWrapperFixed maximo-submitsr-location');
//    		}
//    		else{
//    			$j(this.querySelector('.setLocationWrapper')).addClass('setLocationButtonWrapperFixedDesktop');
//    			$j(this.querySelector('.skipButtonDiv')).addClass('skipbuttonWrapperFixedDesktop maximo-submitsr-location');
//    		}
//    		this.querySelector('.hrborder').style.width = 'calc(100% - 48px)'; // 48px is for padding
//    		if(this.selectedQueryName === 'SERVICEREQUESTROOTLOCATION'){ // when user selected 'All'
//    			if($M.screenInfo.device ==='desktop'){
//    				this.querySelector('.contentWrapper').style.height = 'calc(100% - 57px)'; // 65 is height of one button(without set location button)
//    			}
//    			else{
//    				this.querySelector('.mainWrapper').style.height = 'calc(100% - 65px)';
//    			}
//    		}
//    		else{
//    			if($M.screenInfo.device ==='desktop'){
//    				this.querySelector('.contentWrapper').style.height = 'calc(100% - 109px)'; // 117 is height of two buttons
//    			}
//    			else{
//    				this.querySelector('.mainWrapper').style.height = 'calc(100% - 117px)';
//    			}
//    		}
//    	}
//    	else{ // scroll buttons
//    		$j(this.querySelector('.setLocationWrapper')).removeClass('setLocationButtonWrapperFixed');
//    		$j(this.querySelector('.skipButtonDiv')).removeClass('skipbuttonWrapperFixed maximo-submitsr-location');
//    		$j(this.querySelector('.setLocationWrapper')).removeClass('setLocationButtonWrapperFixedDesktop');
//    		$j(this.querySelector('.skipButtonDiv')).removeClass('skipbuttonWrapperFixedDesktop maximo-submitsr-location');
//    		if($M.screenInfo.device ==='desktop'){
//    			this.querySelector('.contentWrapper').style.height = '100%';
//			}
//			else{
//				this.querySelector('.mainWrapper').style.height = '100%';
//			}
//    		this.querySelector('.hrborder').style.width = '100%';
//    	}
    	
	},
	
	
	_refreshData: function(){
	   
		if(this.filterSite && this.submitInfo && this.stackData){
			this.renderPage(this.submitInfo, this.stackData);
		}
		
	}, 

    /**
	 * Render the data on the page
	 */
 	renderPage: function(submitInfo, stackData) {
		try {

			this.submitInfo= submitInfo;
			this.stackData= stackData;
			var reset = false;
			if(!this.locationFirst){
				if(submitInfo['Asset'] && submitInfo['Asset'].contents && submitInfo['Asset'].contents.assetnum) {
					if(submitInfo['Asset'].contents.location){	// when asset has location
						if(submitInfo['Location'].contents.skipped){
							this.fire('go-back');
						}
						else if(!submitInfo['Location'].contents.skipped && !submitInfo['Location'].contents.selectedinlocation){//asset without location was selected, and location was selected in location step.
							this.assetSiteid = null;
							this.assetOrgid = null;
							this.fire('go-nextdetail',{step:'Location', value: {description:submitInfo['Location'].contents.description,breadcrumb:submitInfo['Location'].contents.breadcrumb,location:submitInfo['Location'].contents.location,skipped: true}});
						}
					}
					else{ // when asset does not have its location info
						this.assetdesc = submitInfo['Asset'].contents.description;
						this.assetnum = submitInfo['Asset'].contents.assetnum;
						this.assetSiteid = submitInfo['Asset'].contents.siteid;
						this.assetOrgid = submitInfo['Asset'].contents.orgid;
					}
				}
				else{
					this.assetdesc = null;
				   this.assetnum = null;
				   if(this.assetSiteid){
					   reset = true;
				   }
				   this.assetSiteid = null;
				   this.assetOrgid = null;
				}
			}
			this.$.nodataLabelWrapper.hidden=true;
		   if(!this.selectedLocation || this.assetSiteid || reset){ // open location step at first time
			   this.$.loadingSpinner.hidden = false;
			   if(this.userLocation && !this.assetSiteid){
					this.getToUserLocation();
					this.addAncestorParams(this.userLocation);
				   this.$.indoorList.hidden = false;
				   this.$.setLocationButton.hidden = false;
				   $j(this.$.indoorList).toggleClass('setLocation',true);
				   $j(this.$.setLocationButtonWrapper).toggleClass('setLocation',true);
				}
				else{
					this._configureLocationList();
				}
		   }
		   else { // get back to location step
			   this.lookupWhereClause = this._getLookupWhereClause();
			   this.selectedLocationDesc = null;
			   this.selectedLocation = null;
			   this.selectedSiteid = null;
			   this.selectedOrgid = null;
			   this.$.locSearch._lookupObject.additionalLookupParams=this.lookupWhereClause;
			   this.$.locSearch.value='';
		   }
   
		   var locationStackData = null;
		   var newStackData = [];
		   if (stackData) {
			   for (var idx = stackData.length -1; idx>=0; idx--) {
				   if (stackData[idx].data && stackData[idx].data.length !== 0) {
					   newStackData.push($M.cloneRecord(stackData[idx]));
					   if (stackData[idx].step == 'Location') {
						   locationStackData = stackData[idx];
					   }
				   }
			   }
   
			   if (locationStackData != null && submitInfo['Location'] && submitInfo['Location'].contents) {
				   this.selectedLocationDesc = submitInfo['Location'].contents.description;
				   this.selectedLocation = submitInfo['Location'].contents.location;
				   this.selectedSiteid = submitInfo['Location'].contents.siteid;
				   this.selectedOrgid = submitInfo['Location'].contents.orgid;
   
				   if (submitInfo['Location'].contents.breadcrumb) {
					   this.selectedBc = submitInfo['Location'].contents.breadcrumb;
				   }
   
				   if (submitInfo['Location'].contents.bcdata) {
					   this.bcdata = submitInfo['Location'].contents.bcdata;
				   }
   
				   if(this.selectedLocation && this.bcdata && this.bcdata.length > 0){
					   this.selectedQueryName = 'SERVICEREQUESTLOCATION';

					   if(this.filterSite){
						 
						  if(this.defaultInsertSite){
								this.addParams('oslc.where='+this.getDefaultSiteidParams());
						  }
					  }

					   this.addParams(this.selectedLocation);
					   this.setBreadCrumb();
				   }
			   }
		   }
		   
		   if (!this.selectedSiteid && submitInfo['Asset'] && submitInfo['Asset'].contents) {
			   this.selectedSiteid = submitInfo['Asset'].contents.siteid;
			   this.selectedOrgid = submitInfo['Asset'].contents.orgid;
		   }
		   
		   this.$.locationstep.setStackList(newStackData);
		} catch(err) {
			this._displayUnexpectedError(err);
		}
 		
	},
	
	/**
	 * To set the values from Location list
	 */
    _configureLocationList: function( ){
		try {
			this.$.setLocationButton.hidden = true;
			this.$.breadcrumb.hidden = true;
			$j(this.$.indoorList).toggleClass('setLocation',false);
			$j(this.$.setLocationButtonWrapper).toggleClass('setLocation',false);
		   this.selectedQueryName = 'SERVICEREQUESTROOTLOCATION';
		   this.additionalparams = [];
		   this.bcdata = [];
		   var param='';
	
		   if(this.filterSite){
			
			if(this.defaultInsertSite){
				if(param !== ''){
					param += this.getDefaultSiteidParams();
				}
				else{
					param += 'oslc.where='+this.getDefaultSiteidParams();
				}
			}
		}else if(this.assetSiteid){
			   if(param !== ''){
				   param += this.getAssetSiteidParams();
			   }
			   else{
				   param += 'oslc.where='+this.getAssetSiteidParams();
			   }
		   }
		   this.additionalparams.push(param);
		   this.additionalparams.push('ignorecollectionref=1');
		   var that = this;
		   this.$.locationcollection.refreshRecords().then(function(collection){
			   if(that.userLocationDesc){
				   that.setYourLocationLabel(that.userLocationDesc);
			   }
			   that.$.loadingSpinner.hidden = true;
		   });
		   this.$.indoorList.hidden = false;
		} catch(err) {
			this._displayUnexpectedError(err);
		}
    	
    },
	/**
	 * To set the values from Location lookup
	 */
    _returnFromLookup: function(e){
		try {
			if(e.detail.records[0] && e.detail.records[0].location){
				this.$.locSearch.value = e.detail.records[0].description;
				this.selectedLocationDesc = e.detail.records[0].description;
				this.selectedLocation = e.detail.records[0].location;
				this.selectedSiteid = e.detail.records[0].siteid;
				this.selectedOrgid = e.detail.records[0].orgid;
				this.$.setLocationButton.hidden = false;
				$j(this.$.indoorList).toggleClass('setLocation',true);
				$j(this.$.setLocationButtonWrapper).toggleClass('setLocation',true);
				this.$.locSearch._lookupObject.additionalLookupParams='responseos=MXAPIOPERLOC&savedQuery=SERVICEREQUESTLOCATION&oslc.where=locancestor.ancestor=%22'+encodeURIComponent(e.detail.records[0].location)+'%22'+this.getDefaultSiteidParams(true);
				
				this.getLocations(e);
				this.$.locSearch.value='';
				
			} else {
				this.$.setLocationButton.hidden = true;
				$j(this.$.indoorList).toggleClass('setLocation',false);
				$j(this.$.setLocationButtonWrapper).toggleClass('setLocation',false);
			}
		} catch(err) {
			this._displayUnexpectedError(err);
		}
	    
	},
	
	

	/**
	 * To check the value from the input field
	 */
	_enableSetLocation : function(e){
		try {
			if(this.selectedLocationDesc===e.currentTarget.dataHost.location){
				this.$.setLocationButton.hidden = false;
				$j(this.$.indoorList).toggleClass('setLocation',true);
				$j(this.$.setLocationButtonWrapper).toggleClass('setLocation',true);
			}
			else{
				this.$.setLocationButton.hidden = true;
				$j(this.$.indoorList).toggleClass('setLocation',false);
				$j(this.$.setLocationButtonWrapper).toggleClass('setLocation',false);
			} 
		} catch(err) {
			this._displayUnexpectedError(err);
		}
       
	},

	_getLookupWhereClause : function() {
		let param='responseos=MXAPIOPERLOC&savedQuery=SERVICEREQUESTLOCATION';

		if(this.filterSite){
			
			if(this.defaultInsertSite){
				param += '&oslc.where='+this.getDefaultSiteidParams();
			}
		}else if(this.assetSiteid){
			param += this.getAssetSiteidParams();
		}

	
		return param;
	},
	
	
	/**
	 * check if a list contains a element
   	 */
	containsElement: function(list , element) {
		try {
			var result = false;
			list.some(function(value){
				if(value == element){
					result = true;
				  }
			  }, this);
		  
		  return result;
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},

  	_handleGetRadiusProperty: function(e) {
  		this.radiusValue = parseInt(e.detail);
  	},
	
	_showNearbyLocations: function() {
		try {
			if ("geolocation" in navigator) {
				$M.toggleWait(true);
				var that = this;
				var geoUtility = document.createElement('maximo-geo-utility');
				var lookupDefinition = JSON.parse(this.$.locSearch.lookup);
				var whereClause = lookupDefinition['additionalLookupParams'] ;
				
				if(whereClause.split('oslc.where').length <=1 ){
					whereClause +='&oslc.where=';
				}
				navigator.geolocation.getCurrentPosition(function(position) {
					console.log(position.coords.latitude);
					console.log(position.coords.longitude);
					$M._currentCoordinates = {"latitude": position.coords.latitude, "longitude": position.coords.longitude};
					var boundingBox = geoUtility.boundingBox(position.coords.latitude, position.coords.longitude, that.radiusValue);
	
					whereClause = whereClause + 'serviceaddress.latitudey%3E%3D' + boundingBox['latMin'] + '%20and%20serviceaddress.latitudey%3C%3D' + boundingBox['latMax'];
					whereClause = whereClause + '%20and%20';
					whereClause = whereClause + 'serviceaddress.longitudex%3E%3D' + boundingBox['lonMin'] + '%20and%20serviceaddress.longitudex%3C%3D' + boundingBox['lonMax'];
					whereClause += that.getDefaultSiteidParams(true);
					
					$M.toggleWait(false);
					$M.showLookup(lookupDefinition['name'], that, that.getAttribute('data-scope-id'), lookupDefinition['href'], null, whereClause);
				}, function(error) {
					$M.toggleWait(false);
					$M.alert($M.localize('uitext', 'mxapisr', 'GeolocationNotAvailable'), $M.alerts.info);
				});
			} else {
				/* geolocation IS NOT available */
				$M.alert($M.localize('uitext', 'mxapisr', 'GeolocationNotAvailable'), $M.alerts.info);
			}
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	_returnQRcode: function(event) {
		try {
			if(event.detail.identifier == 'location'){
				var maximoQbe = new MaximoQbe();
				maximoQbe.setIn('location',event.detail.value);
				if(this.filterSite){
					
					if(this.defaultInsertSite){
						maximoQbe.setEquals('siteid', HTMLEncoder.sanitize(this.defaultInsertSite));
					}
				}
				qbeObject = maximoQbe.toJSON();
				this.$.locationQRCode.queryWhere = qbeObject; 
				var self = this;
				var localizedAll = $M.localize('uitext','mxapisr','all');
				 self.$.locationQRCode.refreshRecords().then(function() {
					 let fakeEvent={};
					 fakeEvent.detail={};
					 if(self.$.locationQRCode.collectionData.length>0){
						 self.bcdata=[];
						 fakeEvent.detail.records=[self.$.locationQRCode.collectionData[0]];
						 fakeEvent.preventDefault  = function(){};
						 self._returnFromLookup(fakeEvent);
					 }else{ 
						  self.$.locSearch.value='';
						 fakeEvent.target={};
						 fakeEvent.target.getAttribute = function(){
							 return localizedAll;
						 }
						 fakeEvent.target.textContent = localizedAll;
						 self.getBcLocation(fakeEvent);
						 $M.notify($M.localize('uitext', 'mxapisr', 'NoResultsFound'), $M.alerts.warn);
					 }
						
				});
				
			}
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
});
