/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2016,2019
 */ 
Polymer({
	is: 'maximo-sr-details-card',
  	behaviors: [BaseComponent,CommonDetailBehavior,ServiceRequestBehavior],
	properties : {
		
		/**
		 * Service Request record
		 */
		record : {
			type : Object,
		},
		
		
		/**
	     * owner of the current registry 
	     */
		collection: {
			type: Object
		},
		
		/**
         * Lightweight representation of the current record 
         */	
    	recordLight:{
    		type: Object,
    		observer:'_loadDetailedRecord'
    	},
		   		
		/**
		 * List of attributes must be loaded
	    */
		attributeList: {
			type: String,
			value: 'status,status_description,status_maxvalue,person._imagelibref,reportedbyname,reportdate_localized,reportdate,ticketid,description,description_longdescription,reportedpriority,reportedpriority_description,targetfinish,ownerperson.displayname,ownerperson._imagelibref,assetnum,asset.assetnum,asset.description,asset._imagelibref,location.description,location.location,location._imagelibref,classstructure.description_class,classstructure._imagelibref,tickect_template.instractions_longdescription,tickect_template.instractions,affectedperson,affectedperson._imagelibref,affectedperson.displayname'
		},
		
		/**
		 * List of attributes used to sort additional details table
	    */
		_additionalDetailsOrderByAttributeNames:{
			type: String,
			value: ''
		
		},
		
		/**
		 * List of attributes must be loaded on additional details table
	    */
		_additionalDetailsAttributeList:{
			type: String,
			value: 'assetattribute.description,mandatory,assetattribute.datatype,alnvalue,numvalue,datevalue,measureunitid,assetattribute.assetattrid,numericdomain.description,alndomain.description'
		},
			
		/**
		 * total additional details 
	    */	
		_allAdditionalDetailsRecordCount:{
			type:Number,
			value: 0
		},
		
		/**
		 * columns displayed on additional details table
	    */
		_additionalDetailsColumns :{
			type: Array,
			value: function (){
				return [
		    		{
						'attribute' : 'displayDescription',
						'sortattribute' : 'assetattrid',
						'title' : $M.localize('uitext', 'mxapisr', 'Description'),
						'sortable' : true, 	
						'flexGrow': '3'
		    		}, 
		    		{
						'attribute' : 'displayValue',
						'sortattribute' : 'alnvalue',
						'title' : $M.localize('uitext', 'mxapisr', 'Response'),
						'sortable' : true,
						'flexGrow': '2'
		    		}
				];
			}
				
		},
		
		
		/**
    	 * Asset record data
    	 */
    	assetrecordData : {
    		type: Object    		
    	},
    	/**
    	 * Asset record URI
    	 */
    	assetURI : {
    		type: Object    		
    	},
    	
    	/**
    	 * Location record data
    	 */
    	locationRecordData : {
    		type: Object    		
    	},
    	
    	/**
    	 * Location record URI
    	 */
    	locationURI : {
    		type: Object    		
    	},
    	
		/**
    	 * attribute type result
    	 */
    	_attributeTypeDomainSet: {
    		type: Object 
    	},
    	
    	/**
    	 * additional Details result
    	 */
    	_additionalDetailsSet: {
    		type: Object 
    	},
    	
    	/**
    	 * attribute type   filter
    	 */
    	_attributeTypeDomainFilter: {
    		type: Array,
    		value: function () {
    			return [{'filtertype': 'SIMPLE', 'field': 'domainid', 'availablevalues': [ {'value': 'DATATYPE', 'selected': true} ]}];
    		} 	
    	},
    	
    	/**
      	 * Indicates if the screen is on desktop mode
      	 */
		isDesktop: {
			type: Boolean,
			value: false,
			notify: true,
			observer: '_updateCSS'
		},
		
		/** total comments */
    	totalcomments: {
   			type: Number
	    },
    	
    	
	},
	
	listeners:{
		'close-long-desc' :'_closeLongDescription',
		'tile-clicked':'_tileClicked',
	},
	
  	created : function(){
  	},
  	
  	ready : function(){
  		var self = this;
  		$j(window).on('resize', function(){
			 self._windowResize();
	    });
  	},
  	
  	attached : function(){
  		var self = this;
		this.async(function(){
			self._windowResize();
		}, 100);   
	},
	
	/**
	 * get the instruction description
	 */
	_getInstructionDescription: function(selectedRecord){
		try {
			if(selectedRecord && selectedRecord.instractions_longdescription) {
				return selectedRecord.instractions_longdescription;
			}
			
			return  this.localize('uitext','mxapibase','NotEntered'); 
		} catch(err) {
			this._displayUnexpectedError(err);
		}
	    
	},
	
	/**
	 * set the scree to desktop / mobile mode
   	 */
	_windowResize : function(){
		this.isDesktop = !$M.isMobile();
	},
	
	/**
	 * set the scree to desktop / mobile mode
   	 */
	_updateCSS : function(){
		try {
			$j(this.$.section_comments).toggleClass('mobile',!this.isDesktop);
		} catch(err) {
			this._displayUnexpectedError(err);
		}
	
	},
	
	
  /**
   * Navigate to appropriate page when tile is clicked
   * When the asset tile in detail page is clicked then it navigates to asset details page fetched from maximo-showasset-done admin component.
   */
	_tileClicked : function(e){
		try {
			if(e.target.id===this.$.assetimg.id && this.record.assetnum){
				e.stopPropagation();
				this.showAssetDetailPage();
			} else 	if(e.target.id===this.$.locationimg.id && this.record.location.location){
				e.stopPropagation();
				this.showLocationDetailPage();
			} 
		} catch(err) {
			this._displayUnexpectedError(err);
		}
	   
   },
   
   
   /**
	 * Fetch Location details and show details page when clicked on location
	 */
	showLocationDetailPage : function(){	
		try {
			var self=this;
			if (!this.locationRecordData || !this.locationRecordData.$alias_this_attr$location || !this.record.$alias_this_attr$location
					||(this.locationRecordData.$alias_this_attr$location!=this.record.$alias_this_attr$location)){
			   $M.toggleWait(true);
				this.locationURI= this.record.href+'/location?oslc.select=location,description,status,type,_imagelibref,hasparent,parent,location_parent.description,location_parent._imagelibref,siteid,orgid,systemid,saddresscode,serviceaddress.streetaddress,serviceaddress.addressline2,serviceaddress.city,serviceaddress.stateprovince,serviceaddress.postalcode,glaccount';
				 this.$.locationresource.loadRecord().then(function(result){
		        	if(result.member && result.member.length>=1){
		        		self.locationRecordData= result.member[0];
		        		self.fire('show-common-location-details',self.locationRecordData);
		        		self.async(function(){
		        			 $M.toggleWait(false);
		               	},100);
		        	} else{
		        	  self.locationRecordData=undefined;
		        	}
			     });
		    } else {
			  this.fire('show-common-location-details',self.locationRecordData);
		    } 
		} catch(err) {
			this._displayUnexpectedError(err);
		}	
		
		
	},
	
	/**
	 * Fetch Asset details and show details page when clicked on assetnum
	 */	
	showAssetDetailPage : function(){
		try {
			this.assetURI= this.record.href+'/asset?oslc.select=assetnum,description,assettype,status,serialnum,siteid,asset_locations.location,asset_locations.description,asset_locations._imagelibref,parent.assetnum,parent.description,parent._imagelibref';
			var self=this;
			if (!this.assetrecordData || (this.assetrecordData.assetnum!=this.record.assetnum)){
				this.$.assetresource.loadRecord().then(function(result){
					$M.toggleWait(true);	
					if(result.member && result.member.length>=1){
							self.assetrecordData= result.member[0];	
							self.fire('show-common-asset-details',self.assetrecordData);
							self.async(function(){
								$M.toggleWait(false);
							},100);
						}else{
						self.assetrecordData=undefined;
						}
				});
			} else {
				this.fire('show-common-asset-details',this.assetrecordData);
			} 
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
		
	
	 /**
	  * Switch sections
	  * 
	  */
	 switchContent: function(event) {		
		try {
			var curIdArray = event.currentTarget.id.split('_');
			var sectionId = 'section_' + curIdArray[curIdArray.length-1];
			this._setLongDescriptioniframeWidth();
			this._showSubTab(sectionId);
			
			
			if(sectionId='section_comments'){
				var comments = $j(this).find('maximo-worklog')[0];
				comments._loadComments(false);
			} 
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
		
	},
	
	/**
	 * switch  panels
	 * set the color of the current tab's parent to the color of the tab.  This ensures the entire page is same color
	 * 
	 */
	_showSubTab:function( sectionId ){
		try {
			var currentTab = this._showHideSection(sectionId);
			//set the color of the current tab's parent to the color of the tab.  This ensures the entire page is same color
			var tabColor = currentTab.style.backgroundColor;
			$j(this).parent().css({'background': tabColor});
			this._currentTab = sectionId;
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	
	
	/**
	 * hide/show  Sections
	 * 
	 */
	_showHideSection: function(sectionName) {
		try {
			this.addContent(sectionName);
			var currentSection;
			$M.publishEvent('page-changed', 'details');
			$j(this.$.maincontainer).children('div').each(function(index, elem) {
				if (elem.id.indexOf(sectionName) > -1) {
					currentSection = this;
				}
				$j(this).hide();
			});
			
			$j(currentSection).show(); 
			
			return currentSection;
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	
	/**
	 * get the text and border color to use for the status badge tile
	 */	
	_getStatusColor: function(status){      // status maxvalue
		var color = '#000000';
	     switch (status){
	          case 'NEW':
	        	  	 color = '#d74108';  // var(--Primary-orange50)
	                 break;
	          case 'INPROG':
	              	 color = '#be9b00';  // var(--Primary-yellow40)
	              	 break;
	          case 'RESOLVED':
	              	 color = '#4b8400';  // var(--Primary-green50)
	              	 break;
	          case 'QUEUED':
	        	  	 color = '#be9b00';  // var(--Primary-yellow40)
	              	 break;
	          case 'CLOSED':
	              	 color = '#000000';  // var(--Primary-gray30)
	              	 break;
	          case 'PENDING':
	        	  	 color = '#be9b00';  // var(--Primary-yellow40)
	              	 break;
	          default:                   
	                 color = '#000000';    // var(--Primary-gray100)
	    }
	    return color;
	},
	
	/**
	 * get the border color to use for the status badge tile
	 */
	_getBorderColor: function(status){      // status maxvalue
		var color = '#000000';		
		switch (status){
		case 'NEW':
            	color = '#ff7832';  // var(--Primary-orange30)
            	break;
         case 'INPROG':
             	 color = '#fdd600';  // var(--Primary-yellow20)
             	 break;
         case 'RESOLVED':
             	 color = '#b4e051';  // var(--Primary-green20)
             	 break;
         case 'QUEUED':
        	 	 color = '#fdd600';  // var(--Primary-yellow20)
             	 break;
         case 'CLOSED':
             	 color = '#aeaeae';  // var(--Primary-gray30)
             	 break;
         case 'PENDING':
        	 	color = '#fdd600';  // var(--Primary-yellow20)
             	 break;
          default:                   
                 color = '#000000';    // var(--Primary-gray100)
	    }
		this._initRTE();
	    return color;
	},
	
	_initRTE: function() {
		try {
			var self = this;
			if(this.record.tickect_template &&  this.record.tickect_template.instractions_longdescription) {
				var instructions = this.record.tickect_template.instractions_longdescription; 
				this.maximoInstructionTile = $M.createElement('maximo-richtexteditor');
				this.maximoInstructionTile.props = {
						'id' : 'instruction',
						'value' :  instructions,
						'readonly': 'true',
						'height': '360px',
						'width': '100%'
					};
				
				$j(this.$.instructionsTileNoData).toggleClass('hidden',true);
			}  else{
				$j(this.$.instructionsTileNoData).toggleClass('hidden',false);
				this.maximoInstructionTile = undefined;
			}	
			if (this.$.divIssueDetailsInstruction.children.length > 0){
				self.$.divIssueDetailsInstruction.removeChild(this.$.divIssueDetailsInstruction.children[0]);
			}
			if(this.maximoInstructionTile){
				this.async(function(){
					self.$.divIssueDetailsInstruction.appendChild(this.maximoInstructionTile);
				},100);
			}
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
		
	},
	
 
 	/**
	 * Fetch icon for status
	 */
	 _getStatusIcon: function(status){
	        return ( status === 'RESOLVED' ? 'Maximo:Confirm': '' ) ;
	 },
	 
 	/**
	 * set the width of Longdescription iframe
	 */
	_setLongDescriptioniframeWidth : function(){   
		try {
			$j('.maximo-sr-details-card> maximo-longdesc iframe').css({'width':window.innerWidth+'px','height':'20px','font-style':'normal'});
			$j('.maximo-sr-details-card> maximo-longdesc iframe').attr('scrolling','no');    
			var longDescplaceholder=$M.localize('uitext', 'mxapibase','LongDescription');
			if(this.$.longEditableDesc.text!==undefined &&  this.$.longEditableDesc.text===longDescplaceholder){
				$j('maximo-longdesc iframe').contents().find('body').css({'font-style':'italic'});
			}
			if(navigator.userAgent.search('Firefox')>=0){
				$j('.maximo-sr-details-card> .longEditableDesc').removeAttr('margin');
				$j('.maximo-sr-details-card> maximo-longdesc iframe').css({'height':'50px','top':'-10px','position':'relative'});
			}
		} catch(err) {
			this._displayUnexpectedError(err);
		} 	
		
	},
	 
 	/**
	 * Flyout for Long description with rich text editor 
	 */	
	_showlongdescflyout : function(e){
		try {
			e.stopPropagation();      	
			var flyoutOwner = this.$.maximoAppHeader.$.main;
			var flyout = $M.showFlyout('ld_flyout', '<div></div>',  e.currentTarget, {'owner':flyoutOwner, 'constrainWidth':true});
			flyout.addButtons([{'label':this.localize('uitext','mxapibase','Close'),'action':true,'event':'close-long-desc','eventTarget':this}]);
			 var maximoRTE = $M.createElement('maximo-richtexteditor');
			 maximoRTE.props = {
				 'id' : 'sr_longdesc',
				 'placeholder' : this.localize('uitext', 'mxapibase','LongDescription'),
				 'readonly': "true",
				 'value' : this.record.description_longdescription?this.record.description_longdescription:'',
				 'width' : '95%'
			 };
			 this.async(function(){
				 var div = $j(flyout.$.internal).find('div');
				div.css({'width':'100%'});
				div.append(maximoRTE);
			 },100);
			 this.LongDescRTE = maximoRTE;
		} catch(err) {
			this._displayUnexpectedError(err);
		}
      
	},
	
	
	
	/**
	 * Suppress flyout without updating the data when the cancel button is clicked 
	 */
	_closeLongDescription : function(e){
		try {
			this.LongDescRTE.innerHTML='';
			this.LongDescRTE.remove();
			this.LongDescRTE = null; 
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	
	
	

	/**
	 * Get Long Description for the work order and set iframe width
	 */
	_getLongDescription : function(longDesc){	
		try {
			this._setLongDescriptioniframeWidth();
			if(longDesc != undefined && longDesc.length > 0 ){
				return longDesc;
			} else{				
				return $M.localize('uitext', 'mxapibase','LongDescription');		
			}	 
		} catch(err) {
			this._displayUnexpectedError(err);
		}			
		
	},
	
	/**
 	 * Handle Back arrow button based on component name
 	 */ 
 	_goBack: function(){
		try {
			var totalComments = $j(this).find('maximo-worklog')[0] ? $j(this).find('maximo-worklog')[0].totalComments : undefined;
			this.fire('hide-sr-details',{record: this.record,
										 attachments: this.$.attachment_tile._attachments.length,
										 comments:totalComments});
		} catch(err) {
			this._displayUnexpectedError(err);
		}
 		
	 },
  	
	/**
	 * Observer for recordLight
	 * Loads full attributes into record 
	 * from mx-resource
	 */
	_loadDetailedRecord: function(){
		try {
			$M.toggleWait(true);  
			this.$.srResource.loadRecord();
			this._loadAdditionalDetails();
			
			var comments = $j(this).find('maximo-worklog')[0];
			if(comments){
				comments._loadComments(false);
			} 
		} catch(err) {
			this._displayUnexpectedError(err);
		}
    	
		
	},
	
	
	/**
	 * Loads additional Details attributes 
	 */ 
	_loadAdditionalDetails: function(){
		try {
			var self = this;
			this.$.attributeTypeDomain.refreshRecords().then(function() {
				self._additionalDetailsOrderByAttributeNames='';
				self.$.additionalDetailsCollection.collectionUri = self.recordLight.href + '/ticketspecclass';
				self.$.additionalDetailsCollection.refreshRecords();
			});
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	
	/**
	 * Loads additional Details attributes 
	 */ 
	_refreshNonPersistentsFields: function(){
		try {
			var NUMERIC;
			var ALN;
			var DATE;
			var measureunit ;
			if(this._attributeTypeDomainSet){
				this._attributeTypeDomainSet[0].synonymdomain.forEach(function(attributeType){
					if(attributeType.maxvalue=='ALN'){
						ALN = attributeType.value;
					}else if(attributeType.maxvalue=='NUMERIC'){
						NUMERIC = attributeType.value;
					}else if(attributeType.maxvalue=='DATE'){
						DATE = attributeType.value;
					}
				}, this);
				
				this._additionalDetailsSet.forEach(function(additionalDetail){
					if(additionalDetail.assetattribute.datatype==ALN){
						
						if(additionalDetail.alndomain && additionalDetail.alndomain.description){
							additionalDetail.displayValue= additionalDetail.alndomain.description;
						}else if(additionalDetail.alnvalue){
							additionalDetail.displayValue= additionalDetail.alnvalue;
						}else{
							additionalDetail.displayValue= this.localize('uitext','mxapibase','NotEntered'); 
						} 
					}else if(additionalDetail.assetattribute.datatype==NUMERIC){
						
						if(additionalDetail.numericdomain && additionalDetail.numericdomain.description){
							additionalDetail.displayValue= additionalDetail.numericdomain.description;
						}else if(additionalDetail.numvalue){
							additionalDetail.displayValue= additionalDetail.numvalue;
						}else{
							additionalDetail.displayValue= this.localize('uitext','mxapibase','NotEntered'); 
						} 
					} else if(additionalDetail.assetattribute.datatype==DATE){
						
						if(additionalDetail.datevalue_localized){
							additionalDetail.displayValue= additionalDetail.datevalue_localized;
						}else{
							additionalDetail.displayValue= this.localize('uitext','mxapibase','NotEntered'); 
						} 
					} 
					measureunit =''
						if(additionalDetail.measureunitid){
							measureunit=' ('+additionalDetail.measureunitid+')';
						} 
					
					if(additionalDetail.assetattribute.description){
						additionalDetail.displayDescription = additionalDetail.assetattribute.description + measureunit;
					}else{
						additionalDetail.displayDescription = additionalDetail.assetattribute.assetattrid + measureunit;
					}
					
				}, this);
			} 
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	
	
	/**
	 * display a record
	 * 
	 */
	showDetail  : function(serviceRequest, displayTab) {
		try {
			this.recordLight = serviceRequest;
			var self = this;
			
			this.async(function() {
				var sectionId = 'section_details';
				self.$.contentSelector.select(0);
				if( displayTab === 'comments'){
					sectionId = 'section_comments';
					this._setLongDescriptioniframeWidth();
					self.$.contentSelector.select(1);
					this._showSubTab(sectionId);	
					if(sectionId='section_comments'){
						var comments = $j(this).find('maximo-worklog')[0];
						comments._loadComments(false);
					}			
				}
				this._showSubTab(sectionId);
			}, 100);
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
		
	},
	
	
	/**
	 * stop event propagation  
	 *  
	 */
	_stopEvent : function(event) {
		event.stopPropagation();
	},
	
	
	/**
	 * woresource mx-resource 
	 * loadRecord success callback
	 */
	_recordLoadSuccessHandler: function(event) {
	    $M.toggleWait(false);
	   
	},
	
	/**
     * woresource mx-resource 
     * loadRecord error callback
     */
	_recordLoadErrorHandler: function(event) {
	    $M.toggleWait(false);
    },
    
    
    /**
	 * woresource mx-resource 
	 * updateRecord success callback
	 */
	_recordUpdatedSuccessHandler: function(event) {
	   $M.toggleWait(false);
	},
	
	
    /**
     * woresource mx-resource 
     * updateRecord error callback
     */
	_recordUpdatedErrorHandler: function(event) {
	    $M.toggleWait(false);
    },
    
    
    /**
     * additionalDetailsCollection maximo-collection
     * refresh success callback
     */
    handleRecordDataRefreshed: function(event) {
    	 this._refreshNonPersistentsFields();
    },
    
    
    /**  
	 * attributeTypeDomain success callback
	 */
    _attributeTypeDomainSuccessHandler: function(event) {
		
	},
	
    
    /**
     * additionalDetailsCollection maximo-collection
     * refresh error callback
     */
    _handleDataError: function(event) {
		if (e.detail && e.detail.Error) {
			$M.alert(e.detail.Error.message);
		}
    },

	/**
     * Check if Tile is readonly
     * Return true if readonly else return false
     */
	_isTileReadOnly: function(record) {
		if(record.status_maxvalue !== 'RESOLVED' 
			&& record.status_maxvalue !== 'CLOSED'){
			return false;
		}
		return true;		
	}
});
