/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2017,2018
 */
Polymer({
  is: 'maximo-plan-item-card',
    behaviors: [BaseComponent, DirtyBehavior],
    listeners: {
    'maximo-lookup-selected' : '_returnFromLookup',
      'maximo-typeahead-selected' : '_returnFromLookup',
      'is-dirty-changed': '_updateBorder',
      'maximo-reset-element':'_resetCard'
    },
  properties : {
    /**
       * the work order record
       */
    record: {
      type : Object,
      observer: '_clearDirty',
      value : null
    },
    /**
       * Interval in seconds to
       * undo delete operation
       */
      undoIntervalSecs: {
        Type: Number,
        value: 3,
        readOnly: true
      },
    /**
     * To get workorder href
     */
    recordhref : {
      type : String
    },
    /**
     * To get Item href
     */
    itemhref : {
      type : String,
      value: ''
    },
    /**
     * To get StoreRoom href
     */
    storeroomhref : {
      type : String,
      value: ''
    },
    /**
     * To get customized href for lookup
     */
      customizeditemref :{
      type : String,
      computed: '_getCustomizedItemRef(record, recordhref)'
    },
    /**
     * Check for item card editable
     */
    editable :{
            type: Boolean,
            observer: '_editableCardFields',
            value : false
    },
    /**
     * To identify new item card.
     */
    newCard : {
      type : Boolean,
      value : false
    },

    /**
          * Flag for material selection
          */
    isMaterialSelected: {
           type: Boolean,
           value: false
    },

    /**
          * Flag for directreq selection
          */
    isDirectreqSelected: {
           type: Boolean,
           value: false
    },

    /**
          * Default itemnum
          */
    itemnum: {
           type: String
    },

    /**
          * Default location
          */
    location: {
           type: String
    },

    /**
          * Default siteid
          */
    siteid: {
           type: String,
           notify: true
    },

    /**
     * To get location clause
     */
    locationClause :{
      type : String,
      computed: '_getLocationClause(siteid,location)'
      },

    /**
     * To get itemnum clause
     */
      itemnumClause :{
      type : String,
      computed: '_getItemnumClause(siteid,itemnum)'
    },

    /**
     * Linetype options from synonym domain
     */
    linetypes :{
      type : String
    },

    /**
     * Restype options from synonym domain
     */
    restypes :{
      type : String
    },

    /**
     * Object for Category domain data
     */
    categoryObject :{
      type : Object
    },

    
    /**
     *  Retrieve location for enable save
     */
    locationid :{
      type : String
    },
    /**
     *  Retrieve the storelocsite for enable save
     */
    storelocsiteid :{
      type : String
    },

    /**
     *  Retrieve the issueto for enable save
     */
    issuetoid :{
      type : String
    },

    /**
     *  Retrieve the company for enable save
     */
    companyid :{
      type : String
    },

    /**
     *  Retrieve the order unit measure for enable save
     */
    orderunitmeasure :{
      type : String
    },	
	/**
     * Default site description
     */
	site: {
      type: Object,
      value: {}
	},
	/**
     * Default value for directIssueFlag
     */
	directIssueFlag :{
		type : Boolean,
		value : true
	},

	/**
	 * Required date
	 */
	requiredate: {
		type : Date,
		value : null
	},

	/* internal time value */
	_time: {
		type: String,
		notify: true
	},
	/* internal date value */
	_date: {
		type: String,
		notify: true
	},
	/* Default Date format*/
	dateFormat : {
		type: String
	},
	
	/* Default Time format */
	timeFormat : {
		type: String
	},
		
	/* internal DateTime foramt*/
	dateTimeFormat : {
		type: String
	}
  },

  attached: function(){
      this.parentResetButton = this.$.divReset.id;
        this.showResetButton= true;
    },

  /**
      * Delete item.
      * Displays overlay on card to undo Delete action.
      */
      _deleteItem: function (e) {
          e.stopPropagation();
          this.fire('disableAddItem', false);
          var self = this;
          var wpitemid;
          var message;
          this.showResetButton=false;
          if (self.record && self.record.wpitemid) {
            wpitemid = self.record.wpitemid;
            if(this.isMaterialSelected){
             message = $M.localize('uitext','mxapiwodetail','wpmaterialdelete');
             }else {
              message = $M.localize('uitext','mxapiwodetail','wpitemdelete');
             }
          } else {
            if(this.isMaterialSelected){
                 message = $M.localize('uitext','mxapiwodetail','wpmaterialdelete');
              }else {
                 message = $M.localize('uitext','mxapiwodetail','wpitemdelete');
              }
              this.fire('disableAddItemButton', true);
          }
          var performDeleteItemOperation = function(overlayNode) {
            var myItemid = null;
            var myHref = null;
            if(self.record){ // the record already existed and is not new
              myItemid = self.record.wpitemid;
                  myHref = self.record.localref;
              }
              if(myItemid){
                self.record.wpitemid = myItemid;
                  self.record.localref = myHref;
              }
              self.fire('deleteItemCard', self.record);
          };

          var undoDeleteItemOperation = function(overlayNode) {
            self.showResetButton=true;
            if (self.newCard ) {
              self.setHighlight(true);
              self.fire('disableAddItem', true);
            }
         };

          $M.createOverlay(this.$.plans_item_card, message, performDeleteItemOperation, undoDeleteItemOperation,
              this.undoIntervalSecs, this.parentElement.parentElement, null);
      },

      /**
      * delete new cards.
      */
      _resetCard: function (e) {
        if (this.newCard ) {
           this._deleteItem(e);
          }
        else {
          this.fire('reset-dirty-element');
        }
      },

      /**
       * To check for card fields editable
       */
   _editableCardFields : function (){
    if(this.editable){
             $j(this.$.plans_item_card).find('maximo-field').each(function(){ this.setAttribute('editable','true') });
             $j(this.$.isrunning).attr('readonly',false);
             $j(this.$.plans_item_card).find('maximo-select').each(function(){ this.setAttribute('editable','true') });
             $j(this.$.plans_item_card).find('maximo-text').each(function(){ this.removeAttribute('readonly') });
        } else{
            $j(this.$.plans_item_card).find('maximo-field').each(function(){ this.removeAttribute('editable') });
            $j(this.$.isrunning).attr('readonly',true);
            $j(this.$.plans_item_card).find('maximo-select').each(function(){ this.removeAttribute('editable') });
            $j(this.$.plans_item_card).find('maximo-text').each(function(){ this.setAttribute('readonly',true) });
        }
		
		if(this.isMaterialSelected){	
			this.changeFlagForRequiredByDate(false);
		}else{
			this.changeFlagForRequiredByDate(true);
		}
   },

    /**
     * To set the values from lookup
     */
    _returnFromLookup: function(e){
      if(e.detail.records[0].taskid !== undefined){
        this.$.plans_item_taskid.value = e.detail.records[0].taskid;
      }
      if(e.detail.records[0].itemnum){
          this.$.plans_item_itemid.value=e.detail.records[0].itemnum;
          this.$.plans_item_description.value = e.detail.records[0].description?e.detail.records[0].description:e.detail.records[0].itemnum;
          if(e.detail.records[0].itemorginfo && e.detail.records[0].itemorginfo.category){
            this.$.plans_item_stockcategory.value = this.getCategoryDescription(e.detail.records[0].itemorginfo.category);
          }
          this.itemhref = e.detail.records[0].href;
          this.itemnum=e.detail.records[0].itemnum;
          if(this.location){
            this.fetchInventoryCost(this.itemnum, this.location, this.siteid);
          }
      }
      if(e.detail.records[0].location){
          this.locationid = e.detail.records[0].location;
          this.$.plans_item_location.value = e.detail.records[0].description?e.detail.records[0].description:e.detail.records[0].location;
        this.storelocsiteid = e.detail.records[0].siteid;
        this.siteid = e.detail.records[0].siteid;
        this.$.plans_item_storelocsite.value = e.detail.records[0].site.description?e.detail.records[0].site.description:this.siteid;
        this.storeroomhref = e.detail.records[0].href;
        this.location= e.detail.records[0].location;
          if(this.itemnum){
            this.fetchInventoryCost(this.itemnum, this.location, this.siteid);
          }
      }
      if(e.detail.records[0] && !e.detail.records[0].location && e.detail.records[0].siteid){
    	  this.$.plans_item_storelocsite.value = e.detail.records[0].description;
    	  this.storelocsiteid = e.detail.records[0].siteid;
    	  this.siteid = e.detail.records[0].siteid; 
      }
      if(e.detail.records[0].personid){
        this.$.plans_item_issueto.value = e.detail.records[0].displayname;
        this.issuetoid =  e.detail.records[0].personid;
      }
      if(e.detail.records[0].company){
        this.$.plans_item_vendor.value = e.detail.records[0].name?e.detail.records[0].name:e.detail.records[0].company;
        this.companyid = e.detail.records[0].company;
      }
      if(e.detail.records[0].measureunitid !== undefined){
        this.$.plans_item_orderunit.value= e.detail.records[0].description?e.detail.records[0].description:e.detail.records[0].measureunitid;
        this.orderunitmeasure=e.detail.records[0].measureunitid;
      }
    },

     /**
    * Enable/disable save button for required item card fields.
    */
   _enableSave : function(e){
	   var self = this;
       if(this.directIssueFlag){
    	   this.directIssueFlag =  false;
		   this.async(function(){
	       if(!this.isMaterialSelected){
	         this.$.isrunning.readonly=false;
	         $j(this.$.isrunning).find('.switch').removeAttr('style');
	       } else {
	         this.$.isrunning.readonly=true;
	         $j(this.$.isrunning).find('.switch').css('background','#aeaeae');
	       }
	        if(!this.newCard){
	          this.clearDirtyStatus();
	        }
		   },600); 
	   }
         this.async(function(){
           var itemid = this.$.plans_item_itemid.value;
           var itemquanitity = this.$.plans_item_itemqty.value;
           var itemunitcost = this.$.plans_item_unitcost.value;
           var itemdescription = this.$.plans_item_description.value;
           var materialdescription = this.$.plans_material_description.value;
           var location = this.locationid;
           var storelocsite = this.storelocsiteid;
	         var requiredbydate;
	         if(this.record && this.record.wpitemid){
	        	 requiredbydate = $j('#plans_item_requiredate_'+this.record.wpitemid).val();
	         }else {
	        	 requiredbydate = $j('#plans_item_requiredate_').val();
	         }

           this.location = location;
           this.itemnum = itemid;

           if(itemid===undefined){
             this.itemhref = undefined;
           }

           if(location===undefined){
             this.storeroomhref = undefined;
           }

           if(storelocsite && storelocsite!==self.siteid ){
             self.siteid = storelocsite;
           }else {
             this.storelocsiteid = self.siteid;
           }

           //Execute only when lineType is modified
           if(e.target && e.target.nodeName === 'MAXIMO-FIELD' && (e.target.id==='plans_item_card_plans_item_linetype' || e.target._id==='plans_item_card_plans_item_linetype' || e.target.id==='plans_item_linetype')) {
             var lineTypeSelected = e.target.value;
         if(lineTypeSelected !== undefined){
           this.$.plans_item_restype_select.value='';
           this.$.plans_item_issueto.value='';
           this.issuetoid='';
           if(this.domHost.getInternalValueForLinetype(lineTypeSelected) === 'MATERIAL') {
              this.$.plans_item_itemqty.value='';
              this.$.plans_item_linecost.value='';
              this.$.plans_item_unitcost.value=0;
              this.$.plans_item_itemid.value='';
              this.$.plans_item_description.value = '';
              this.$.plans_material_description.value = '';
              this.$.plans_item_location.value = '';
              this.location = '';
              this.isMaterialSelected = true;
              this.$.plans_item_directreq.value = true;
              this.$.plans_item_stockcategory = '';
              //Disable fields
              this.$.plans_item_location.removeAttribute('editable');
              this.$.isrunning.checked = true;
              $j(this.$.isrunning).attr('readonly',true);
			  this.changeFlagForRequiredByDate(false);
              this.isDirectreqSelected = true;
           }else {
             this.$.plans_item_itemqty.value='';
             this.$.plans_item_linecost.value='';
             this.$.plans_item_unitcost.value=0;
             this.isMaterialSelected = false;
             this.isDirectreqSelected = false;
              this.$.plans_item_description.value = '';
              this.$.plans_material_description.value = '';
              //Enable Fields
             this.$.plans_item_location.setAttribute('editable','true');
             $j(this.$.isrunning).attr('readonly',false);
             this.$.isrunning.checked = false;
             this.changeFlagForRequiredByDate(true);
           }
           }
           }
           var directissue = this.$.isrunning.checked;
          //set default value to ITEM
           if(this.domHost && !this.$.plans_item_linetype_select.value){
             this.$.plans_item_linetype_select.value = this.domHost.getExternalValueForLinetype('ITEM');
           }
           //set default value to AUTOMATIC
           if(this.domHost && !this.$.plans_item_restype_select.value){
             this.$.plans_item_restype_select.value = this.domHost.getExternalValueForRestype('AUTOMATIC');
           }
           if (this.record && !this.newCard && !this.isDirty){
             if (this.$.plans_item_linetype_select.maxValue===this.record.linetype){
               this.$.plans_item_linetype_select.value = this.$.plans_item_linetype_select.maxValue;
             }
             if (this.$.plans_item_restype_select.maxValue===this.record.restype){
               this.$.plans_item_restype_select.value = this.$.plans_item_restype_select.maxValue;
             }
             this.clearDirtyStatus();
           }
           this.lineType = this.$.plans_item_linetype_select.value;
           this.resType = this.$.plans_item_restype_select.value;
           if(this.domHost && this.lineType && this.domHost.getInternalValueForLinetype(this.lineType) === 'MATERIAL'){
             this.isMaterialSelected = true;
             $j(this.$.isrunning).attr('readonly',true);
           } else if(this.domHost && this.lineType && this.domHost.getInternalValueForLinetype(this.lineType) === 'ITEM'){
             this.isMaterialSelected = false;
             if(this.editable && !directissue){
                this.$.plans_item_location.setAttribute('editable','true');
                $j(this.$.isrunning).attr('readonly',false);
				this.changeFlagForRequiredByDate(true);
             }
           }

           if(this.editable ) {
             if(directissue) {
               this.$.plans_item_restype.removeAttribute('editable');
               this.isDirectreqSelected = true;
               
               if (this.record && !this.newCard && !this.isDirty){
            	   this.$.plans_item_restype.value=this.domHost.getExternalValueForRestype('AUTOMATIC');
            	   this.clearDirtyStatus();
               }else {
            	   this.$.plans_item_restype.value=this.domHost.getExternalValueForRestype('AUTOMATIC');
               }
               this.$.plans_item_vendor.setAttribute('editable','true');
               this.$.plans_item_location.removeAttribute('editable');
             }else {
               this.$.plans_item_restype.setAttribute('editable','true');
               this.isDirectreqSelected = false;
               this.$.plans_item_vendor.removeAttribute('editable');
             }
           }

	    	  var siteDescription = this.$.plans_item_storelocsite.value;
	    	  if(!siteDescription || siteDescription===''){
	        	  this.storelocsiteid = ' ';
	        	  this.siteid = ' ';
	        	  this.locationid = ' ';
	        	  this.location = ' ';
	        	  if(!this.itemnum){
	        		  this.itemnum = ' ';
	        	  }
	    	  }
	      
	    	  var locationDescription = this.$.plans_item_location.value;
	    	  if(!this.isDirectreqSelected && (!locationDescription || locationDescription==='')){
	    		  this.locationid = ' ';
	        	  this.location = ' ';
	        	  location = ' ';
	        	  if(this.domHost && this.$.plans_item_linetype_select.value === this.domHost.getExternalValueForLinetype('ITEM')){
	    			  this.$.plans_item_unitcost.value = 0.0;
	    		  }
	    	  }
	    	  var itemDescription = this.$.plans_item_description.value;
	    	  if(!itemDescription || itemDescription===''){
	    		  this.itemnum = ' ';
	    		  if(this.domHost && this.$.plans_item_linetype_select.value === this.domHost.getExternalValueForLinetype('ITEM')){
	    			  this.$.plans_item_unitcost.value = 0.0;
	    		  }
	    		  this.$.plans_item_itemid.value = '';
	    	  }

	    	  
	    	  //Logic to enable or disable Save button
             if(this.domHost && this.lineType && this.domHost.getInternalValueForLinetype(this.lineType) === 'ITEM'){
               if((itemdescription && itemdescription.length !== 0 && itemid && itemid.length !== 0 && itemquanitity > 0) &&
                   ((!directissue && location && location.trim().length !==0) ||directissue)){
                   this.$.saveitemcardDivfront.firstElementChild.disabled = false;
                   this.$.saveitemcardDivback.firstElementChild.disabled = false;

               } else {
                 this.$.saveitemcardDivfront.firstElementChild.disabled = true;
                 this.$.saveitemcardDivback.firstElementChild.disabled = true;
               }
             } else if(this.domHost && this.lineType && this.domHost.getInternalValueForLinetype(this.lineType) === 'ITEM' ){
               if(itemdescription && itemdescription.length !== 0 && itemid && itemid.length !== 0 && itemquanitity > 0){
                   this.$.saveitemcardDivfront.firstElementChild.disabled = false;
                   this.$.saveitemcardDivback.firstElementChild.disabled = false;

               } else {
                 this.$.saveitemcardDivfront.firstElementChild.disabled = true;
                 this.$.saveitemcardDivback.firstElementChild.disabled = true;
               }
             }
              else{
                 if(materialdescription && materialdescription.length !== 0 && itemquanitity > 0){
                   this.$.saveitemcardDivfront.firstElementChild.disabled = false;
                     this.$.saveitemcardDivback.firstElementChild.disabled = false;
                 } else {
                   this.$.saveitemcardDivfront.firstElementChild.disabled = true;
                   this.$.saveitemcardDivback.firstElementChild.disabled = true;
                 }
               }
              var that = self;
              this.async(function(){
             	  if(this.isMaterialSelected){
                	  that.$.isrunning.readonly=true;
                      $j(that.$.isrunning).find('.switch').css('background','#aeaeae');
                  } else {
                   	  if(!that.editable){
	                	  that.$.isrunning.readonly=true;
                	  } else {
                   	  	  that.$.isrunning.readonly=false;
                      }
                      $j(that.$.isrunning).find('.switch').removeAttr('style');
                   }
              },900);
         },100);
   },

     /**
        * To Save/Update item Card
        */
       _saveItem: function (e) {
          var itemData = {};
          var message;
          var self = this;
         if(this.record && this.record.href){
          itemData = this.record;
         itemData._action='Update';
         if(this.isMaterialSelected){
           message = $M.localize('uitext','mxapiwodetail','wpmaterialsave');
         }else {
           message = $M.localize('uitext','mxapiwodetail','wpitemsave');
         }
         if(this.resType){
           itemData.restype=this.resType;
         }
         itemData.linetype=this.lineType;
         if(this.domHost.getInternalValueForLinetype(itemData.linetype)==='ITEM') {
           itemData.description=this.$.plans_item_description.value;
           itemData.orderunit='';
         } else {
           itemData.description=this.$.plans_material_description.value;
         }
         itemData.directreq=this.$.plans_item_directreq.value;

         if(itemData.directreq){
           itemData.location='';
             itemData.storelocsite='';
             itemData.restype=this.domHost.getExternalValueForRestype('AUTOMATIC'); //Setting it to automatic when directreq
             itemData.vendor=this.companyid;
         }else {
           itemData.location=this.locationid;
             itemData.storelocsite=this.storelocsiteid;
             itemData.vendor='';
         }
         if(this.$.plans_item_itemqty.value){
           itemData.itemqty=parseFloat(this.$.plans_item_itemqty.value);
         }

         itemData.unitcost=parseFloat(this.$.plans_item_unitcost.value);
         itemData.linecost=itemData.unitcost*itemData.itemqty;

         if(this.$.plans_item_taskid){
           itemData.taskid = parseInt(this.$.plans_item_taskid.value);
         }
         itemData.issueto=this.issuetoid;

         } else {
           itemData._action='Add';
           if(this.isMaterialSelected){
                message = $M.localize('uitext','mxapiwodetail','wpmaterialsave');
              }else {
                 message = $M.localize('uitext','mxapiwodetail','wpitemsave');
              }
           itemData.linetype=this.$.plans_item_linetype_select.value;
           if(this.domHost && this.domHost.getInternalValueForLinetype(itemData.linetype)==='ITEM') {
             itemData.description=this.$.plans_item_description.value;
           } else {
             itemData.description=this.$.plans_material_description.value;
             itemData.orderunit=this.orderunitmeasure;
           }

           itemData.itemnum=this.$.plans_item_itemid.value;
           if(this.$.plans_item_taskid){
           itemData.taskid = parseInt(this.$.plans_item_taskid.value);
         }
           if(this.$.plans_item_location.value){
             itemData.location=this.locationid;
           }
           if(this.$.plans_item_storelocsite.value){
             itemData.storelocsite=this.storelocsiteid;
           }
           itemData.issueto=this.issuetoid;
           if(this.$.plans_item_restype_select.value){
             itemData.restype=this.resType;
           }
           itemData.directreq=this.$.plans_item_directreq.value;
           if(itemData.directreq){
             itemData.location='';
               itemData.storelocsite='';
               itemData.vendor=this.companyid;
               itemData.restype=this.domHost.getExternalValueForRestype('AUTOMATIC'); //Setting it to automatic when directreq
           }else {
             itemData.location=this.locationid;
               itemData.storelocsite=this.storelocsiteid;
               itemData.vendor='';
           }

           if(this.$.plans_item_itemqty.value){
             itemData.itemqty=parseFloat(this.$.plans_item_itemqty.value);
           }

           itemData.unitcost=parseFloat(this.$.plans_item_unitcost.value);
           itemData.linecost=itemData.unitcost*itemData.itemqty;
          }
         if(this._date || this._time){
			  if(!this._time){
				  this._time = '00:00';
			  }
			  this.setDefaultDateTimeFormat();
			  var tempDateTimeFormat = this.dateTimeFormat;
			  if($M.isMobile()){//Update date format to ISO for mobiles
				  tempDateTimeFormat = 'YYYY-MM-DD HH:mm';
	  	  	  }	  	  	  
			  if($M.isMobile()){
					itemData.requiredate = moment(this._date + " " + this._time,tempDateTimeFormat).toDate();
				}else{
			  		itemData.requiredate = this._date + " " + this._time;
				}
			  if((this._date || this._time) && itemData.requiredate=='Invalid Date'){
				  itemData.requiredate = this._date + " " + this._time;
			  }
		   }

          this.flipToFront(e);
          self.fire('saveItemCard', itemData);
       },
    /**
       * This method will toggle border when card is in edit mode
       */
      _updateBorder: function() {
          this.setHighlight(this.isDirty);
      },

      /**
       * This method will toggle border when card is in edit mode
       */
      setHighlight: function(isHighlight) {
          $j(this.$.plans_item_innerDiv).toggleClass('dirtyBorder',isHighlight);
      },

    /**
       * Reset the values.
       */
    _clearDirty: function(e){
		this._time = '';
			this._date = '';
		if(this.record && this.record.requiredate){
			this.setRequiredByDate(this.record.requiredate_localized, true);
		} else{
			//get current date and time
			var self = this;
	        $M.getServerDateTime().then(function(response){
	        	self.setRequiredByDate(response, false);
	        	self.clearDirtyStatus();
	        });
		}
      this.clearDirtyStatus();
    },
    
    /**
     * Set Date based on format.
     */    
     setRequiredByDate: function (dateInput, hasReqDate){
    	var dateFormat = $M.getMaxauth().whoami.dateformat.toUpperCase();
		var timeFormat = $M.getMaxauth().whoami.timeformat;
		var dateTimeFormat = $M.getMaxauth().whoami.datetimeformat.replace($M.getMaxauth().whoami.dateformat,$M.getMaxauth().whoami.dateformat.toUpperCase());
    	if($M.isMobile()){
			//INPUT element needs date/time value in ISO formats
    		if(hasReqDate){
    			this._date = moment(dateInput,dateFormat).format('YYYY-MM-DD'); 
    	  		this._time = moment(dateInput,dateTimeFormat).format('HH:mm');
    		}else{
    			this._date = moment(dateInput.datetime,dateFormat).format('YYYY-MM-DD');		    	  		
				this._time = moment(dateInput.time,timeFormat).format('HH:mm');
    		}
	  	}else {
	  		if(hasReqDate){ 			
				this._date = moment(dateInput,dateFormat).format(dateFormat);	
				this._time = moment(dateInput,dateTimeFormat).format(timeFormat);
	  		}else{
	  			this._date = moment(dateInput.datetime,dateFormat).format(dateFormat);	
		  		this._time = dateInput.time.toUpperCase();
	  		}
	  	}
		if(this.record && this.record.requiredate){
			this.requiredate = this.record.requiredate;
		}
	},
	
    /**
     * flip the card to back.
     */
      flipCard: function(e) {
          e.stopPropagation();
          e.preventDefault();
          var flipper = $M.findElementParent(e.target, 'MAXIMO-FLIPPER');
          if (!flipper) {
              flipper = e.currentTarget;
          }
          if(this.$.isrunning.checked){
              this.$.plans_item_restype.editable = false;
          }
          flipper.showBack();
      },

    /**
     * flip the card to front.
     */
      flipToFront: function(e) {
          e.stopPropagation();
          e.preventDefault();
          var flipper = $M.findElementParent(e.target, 'MAXIMO-FLIPPER');
          if (!flipper) {
              flipper = e.currentTarget;
          }
          flipper.showFront();
      },

      /**
       * To fetch the Items for lookup.
       */
    _getCustomizedItemRef : function(itemRecord, recordhref){
      if(itemRecord && itemRecord.localref !== undefined){
          return itemRecord.localref;
        }
        return recordhref+'/showplanmaterial/zombie';
    },

    /*
     * Get Location clause for Item Lookup
     */
    _getLocationClause : function(siteid,locationselected){
    	siteid = siteid.trim();
    	locationselected = locationselected.trim();
		var setuData = '';
		if(siteid && siteid!==''){
			setuData = "\'storelocsite\':\'"+siteid+"\'";
		}
		if(locationselected && locationselected!==''){
			if(setuData!==''){
				setuData = setuData +',';
			}
			setuData = setuData +"\'location\':\'"+locationselected+"\'";
		}
		
		if(setuData!==''){
				setuData = "setupdata={\'wpmaterial\':{"+setuData+"}}";
		}
     
      return setuData;
    },

    /*
     * Get Itemnum clause for location Lookup
     */
    _getItemnumClause : function(siteid,itemnumselected){
	  	siteid = siteid.trim();
	  	itemnumselected = itemnumselected.trim();
		var setuData = '';
		if(siteid && siteid!==''){
			setuData = "\'storelocsite\':\'"+siteid+"\'";
		}else {
			setuData = "\'storelocsite\':\'\'";
		}
		if(itemnumselected && itemnumselected!==''){
			if(setuData!==''){
				setuData = setuData +',';
			}
			setuData = setuData +"\'itemnum\':\'"+itemnumselected+"\'";
		}	
		if(setuData!==''){
				setuData = "setupdata={\'wpmaterial\':{"+setuData+"}}";
		}
	    return setuData;
    },

      /**
       * This method will stop navigation when card is in edit mode
       */

      _disableNavigation: function() {
          if(this.isDirty){
            $M.toggleDisabled([ $M.workScape.querySelector('#hamburger'), $M.workScape.querySelector('#sideMenu'), $M.workScape.querySelector('#details_maximoAppHeader_main'), $M.workScape.querySelector('#details_supervisorPlans_plansSelector'), $M.workScape.querySelector('#workscape_user')], true);
          }else{
            $M.toggleDisabled([ $M.workScape.querySelector('#hamburger'), $M.workScape.querySelector('#sideMenu'), $M.workScape.querySelector('#details_maximoAppHeader_main'), $M.workScape.querySelector('#details_supervisorPlans_plansSelector'), $M.workScape.querySelector('#workscape_user')], false);
          }
       },

    /**
     * _showInfo will show the Tooltip.
    */
    _showInfo: function(e) {
      e.stopPropagation();
      var tooltip = this.$.info;
      $M.showTooltip(tooltip, e.currentTarget);

    },

    /**
     * fetch Item cost based on itemnum, location and storelocsite.
    */

    fetchInventoryCost: function(itemnum,location,storelocsite) {
      if(itemnum && itemnum!=='' && location && location!=='' && storelocsite && storelocsite!=='') {
            var resourceURL = this.recordhref+ '/showplanmaterial';
            this.collectionURI = resourceURL;
            this.contentLocalized = true;
            if(this.domHost.$.wpmaterial.creationInfo){
            	this.domHost.$.wpmaterial.creationInfo[0].href = resourceURL;
            }else{
            	this.domHost.$.wpmaterial.creationInfo=[{'href':resourceURL}];
            }
        var props = '*';
        var items = [];
        var itemData = {};
        itemData._action='Add';
        itemData.location=location;
        itemData.itemnum=itemnum;
        itemData.itemqty=1.0;
        itemData.storelocsite=storelocsite;
        items.push(itemData);

        var planitemcardreference = this;
        this.domHost.$.wpmaterial.createRecord(items, props, true).then( function(apiresponse) {

          if(apiresponse && apiresponse.response && apiresponse.response.wpmaterial && apiresponse.response.wpmaterial.length>0 && apiresponse.response.wpmaterial[0].unitcost) {
            var tempItemCost = apiresponse.response.wpmaterial[0].unitcost;
            planitemcardreference.$.plans_item_unitcost.value = tempItemCost;

          }
        });
      }
    },
    /**
     * Enable card hover based on editable property
     */
    _enableCardHover: function(e){
      if(this.editable){
        $j(e.currentTarget).parents('#plans_item_card_plans_item_innerDiv').toggleClass('cardLayout_item_hover',true);
      }else{
        $j(e.currentTarget).parents('#plans_item_card_plans_item_innerDiv').toggleClass('cardLayout_item_hover',false);
      }
    },

    /**
     * Method to return description and set id for enable save
     */
    _getDataLabel: function(desc,id,type){
      if(type === 'LOCATION'){
        this.locationid = id;
      }else if(type === 'STORELOCSITE'){
        this.storelocsiteid = id;
      }else if(type === 'ISSUETO'){
        this.issuetoid = id;
      }
      return desc;
    },
    
		/**
		 * Mark required or not for requiredbydate attribute on the UI
		 */
		changeFlagForRequiredByDate(flag){
			if(this.record){
				  $j('#plans_item_requiredate_'+this.record.wpitemid).attr('required',flag);
				}else{
					$j('#plans_item_requiredate_').attr('required',flag);
			}			
		},

    /**
     * Method to return description for the category
     */
    getCategoryDescription: function(category, categoryObj){
    	var description = category;
    	if(this.categoryObject && this.categoryObject[category]){
    		description = this.fixDescription(category,this.categoryObject[category].description);
    	}
    	
    	return description;
    },
    /**
     * Handle the display of storeroom location description and location.
     */
    _getLocation : function(materiallocation){
    	if(materiallocation){
    		this.locationid = materiallocation.location;
    		return materiallocation.description?materiallocation.description:materiallocation.location;
    	}
    },

	/** 
	 * Set the date format
	 */ 
 	setDefaultDateTimeFormat : function(){
		if ($M.getMaxauth().whoami && $M.getMaxauth().whoami.dateformat && $M.getMaxauth().whoami.timeformat && $M.getMaxauth().whoami.datetimeformat){
	 		this.dateFormat = $M.getMaxauth().whoami.dateformat.toUpperCase();	  			
	  		this.dateTimeFormat = $M.getMaxauth().whoami.datetimeformat.replace($M.getMaxauth().whoami.dateformat,$M.getMaxauth().whoami.dateformat.toUpperCase());
	  		this.timeFormat =  $M.getMaxauth().whoami.timeformat;
	  		this.showAMPM = $M.getMaxauth().whoami.datetimeformat.includes('a');
		} else {
			this.dateFormat = 'MM/DD/YYYY';
			this.dateTimeFormat = 'MM/DD/YYYY hh:mm';
			this.timeFormat = 'hh:mm';
			this.showAMPM = true;
		}
	},
		
	/**
     * parse date for display date only
     */
    _parseDate: function(newValue){
        var dateVal = '';
        if (newValue){
            if(!this.dateFormat){
                 this.setDefaultDateTimeFormat();
            }
            dateVal = moment(newValue).format(this.dateFormat);                      
        }
        return dateVal;
     },
     
     /**
      * parse date for display time only
      */
      _parseTime: function(newValue){
         var timeVal = '';
         if (newValue){
          	var newDateValue = new Date(newValue);
            if(!this.dateFormat){
                this.setDefaultDateTimeFormat();
            }
            if(this.showAMPM){
            	timeVal = moment(newDateValue).format('hh:mm A');
            } else {
            	timeVal = moment(newDateValue,this.dateTimeFormat).format('HH:mm');
            }                          
         }
         return timeVal ;
      },
      /**
	 	* Show id if description is unspecified
	 	*/
	     _validateDesc: function(record,indentifier){
	 		if(record && record.company && record.company.company && indentifier==='companyName'){
	 			return this.fixDescription(record.company.company,record.company.name);
	 		}else if(record && record.storelocsite && indentifier==='storelocsite'){
	 			return this.fixDescription(record.storelocsite,record.site.description);
	 		}else if(record && record.orderunit && indentifier==='measureunit'){
	 			return this.fixDescription(record.orderunit,record.measureunit.description);
	 		}
	     }
});
